<?php

/**
 * @file test.instance.inc
 * Test the build scripts
 */

/**
 * Implements drush_hook().
 */
function drush_instance_test() {
  // Include our test aliases
  drush_set_context('ALIAS_PATH', array(dirname(__FILE__) . '/tests'));

  // Bring in @instance.local, but fiddle with its root directory
  drush_sitealias_get_record('@instance.local');
  $aliases =& drush_get_context('site-aliases');
  $site_alias =& $aliases['@instance.local'];
  $site_alias['root'] = $site_root = _drush_instance_tmp_unique();
  // Write it out, so that the separate processes can see it
  drush_mkdir($aliases_dir = _drush_instance_tmp_unique());
  file_put_contents("$aliases_dir/aliases.drushrc.php",
    "<?php \$aliases['instance.local'] = " . var_export($site_alias, TRUE) . ";");

  // Need to set umask for all child processes, so that we can detect
  // that permissions-wrangling has really worked
  umask(022);

  // Invoke instance-build as a separate process
  drush_log("RUN : build, as a separate process. Please wait.", "ok");
  $success = drush_shell_exec("drush instance-build @instance.local --alias-path=$aliases_dir");
  print join("\n", drush_shell_exec_output()) . "\n";
  if (!$success) {
    return drush_set_error("Quitting because of errors in sub-process.");
  }

  // Tests:
  drush_log("TEST: build", "ok");
  // 1. Drupal 7.21 downloaded
  if (file_exists("$site_root/CHANGELOG.txt") &&
     strpos(file_get_contents("$site_root/CHANGELOG.txt"),"Drupal 7.21, 2013-03-06") !== false) {
    drush_log("Found Drupal 7.21 CHANGELOG.txt", "success");
  }
  else {
    return drush_log("Cannot find Drupal 7.21 CHANGELOG.txt", "error");
  }
  // 2. sites/default is git repos and contains what we expect
  if (count(glob("$site_root/sites/default/instance.drush.inc"))) {
    drush_log("Found contents of sites/default", "success");
  }
  else {
    return drush_log("Cannot find example file in sites/default", "error");
  }

  // 3. sites/default has the correct git origin repository
  drush_shell_cd_and_exec("$site_root/sites/default", 'git remote show origin 2>' . drush_bit_bucket());
  $output = drush_shell_exec_output();
  foreach($output as $line) {
    if(trim($line) == "Fetch URL: http://git.drupal.org/project/drush_instance.git") {
      $found_origin = TRUE;
      drush_log("Found origin of sites/default git repository", "success");
    }
  }
  if (empty($found_origin)) {
    return drush_log("Could not find origin of sites/default git repository", "error");
  }

  // 4. sites/default is on the right branch
  drush_shell_cd_and_exec("$site_root/sites/default", 'git branch 2>' . drush_bit_bucket());
  $output = drush_shell_exec_output();
  if (!empty($output) && in_array("* 7.x-1.x-dev", $output)) {
    drush_log("sites/default git repository is on 7.x-1.x-dev", "success");
  }
  else {
    return drush_log("Could not detect sites/default git repository to be on 7.x-1.x-dev", "error");
  }

  // 5. ./user_module_symlink is a symlink to the user module
  $symlink = "$site_root/symlinks/user";
  if (!file_exists($symlink) || !is_link($symlink)) {
    return drush_set_error("Symlink not found: $symlink");
  }
  if (!in_array("$symlink/user.module", glob("$symlink/*"))) {
    return drush_set_error("Symlink did not work (can't see contents): $symlink");
  }

  // Ensure permissions
  // 6. sites/files/core - chmod_group
  if (!(fileperms("$site_root/sites/files/core") & 0x0010)) {
    return drush_set_error("sites/files/core has not had permission change chmod_group");
  }
  // 2. some_random_toplevel_file.txt - chmod_other
  if (!(fileperms("$site_root/some_random_toplevel_file.txt") & 0x0002)) {
    return drush_set_error("some_random_toplevel_file.txt has not had permission change chmod_group");
  }

  // Invoke instance-build --rebuild as a separate process
  drush_log("RUN : rebuild, as a separate process. Please wait.", "ok");
  $success = drush_shell_exec("drush instance-build @instance.local --rebuild --alias-path=$aliases_dir");
  print join("\n", drush_shell_exec_output()) . "\n";
  if (!$success) {
    return drush_set_error("Quitting because of errors in sub-process.");
  }

  // Tests:
  drush_log("TEST: rebuild", "ok");
  // 1. Check files preserved
  _drush_instance_build_validate_paths($site_alias);
  foreach($site_alias["instance"]["paths"] as $path => $config) {
    if (file_exists("$site_root/$path")) {
      if (!$config['preserve_on_rebuild']) {
        return drush_set_error("Path $path should not have been preserved by rebuild, but it's still there!");
      }
      drush_log("Preserved during rebuild: $path", "success");
    }
    else {
      if ($config['preserve_on_rebuild']) {
        return drush_set_error("Path $path should have been preserved by rebuild, but it was lost!");
      }
      drush_log("Discarded during rebuild: $path", "success");
    }
  }

  // Invoke instance-build --branch=master as a separate process
  drush_log("RUN : rebuild --branch=master, as a separate process. Please wait.", "ok");
  $success = drush_shell_exec("drush instance-build @instance.local --branch=master --rebuild --alias-path=$aliases_dir");
  $output = drush_shell_exec_output();
  // Command should fail
  if ($success) {
    return drush_set_error("Was able to build off branch master; but there should be no makefile on that branch");
  }
  // It should fail based on not finding the makefile
  if(strpos(join($output, ""), "Makefile not present in repository.")
       === FALSE) {
    return drush_set_error("Was able to find a makefile in branch master");
  }
  drush_log("Attempting to build on a different branch failed correctly", "success");

  // Invoke instance-build --tag=not_a_tag as a separate process
  drush_log("RUN : build --tag=not_a_tag, as a separate process. Please wait.", "ok");
  $success = drush_shell_exec("drush instance-build @instance.local --rebuild --tag=not_a_tag --alias-path=$aliases_dir");
  $output = drush_shell_exec_output();
  // Command should fail
  if ($success) {
    return drush_set_error("Was able to build off a tag that does not exist");
  }
  // It should fail based on not finding the makefile
  if(strpos(join($output, ""), "Unable to check out tag not_a_tag")
       === FALSE) {
    return drush_set_error("Was able to find a makefile in a tag that does not exist");
  }
  drush_log("Attempting to build on a bad tag failed correctly", "success");
}
